#ifndef _PTGGEN_H
#define _PTGGEN_H

#include <stdio.h>
#include "eliproto.h"

/* Include-files specified in .ptg.phi-files */

#include "ptg.h"
/* Definition of Output-Macros */

#if !defined(PTG_OUTPUT_FILE)
#    define PTG_OUTPUT_FILE FILE *
#    define PTG_OUTPUT_DEFAULT 1
#else
#    define PTG_OUTPUT_DEFAULT 0
#endif
#if !defined(PTG_OUTPUT_STRING)
#    define PTG_OUTPUT_STRING(file,param) fputs(param, file)
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    fprintf(file,"%d",param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   fprintf(file,"%ld",param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  fprintf(file,"%d",(int) param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   fputc(param, file)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  fprintf(file,"%g",(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) fprintf(file,"%g",param)
#    endif

#else
     /* PTG_OUTPUT_STRING defined */
     extern void _PTGPrintInt ELI_ARG((PTG_OUTPUT_FILE,int));
     extern void _PTGPrintLong ELI_ARG((PTG_OUTPUT_FILE,long));
     extern void _PTGPrintChar ELI_ARG((PTG_OUTPUT_FILE,char));
     extern void _PTGPrintDouble ELI_ARG((PTG_OUTPUT_FILE,double));
#    if !defined(PTG_OUTPUT_INT)
#            define PTG_OUTPUT_INT(file,param)    _PTGPrintInt(file,param)
#    endif
#    if !defined(PTG_OUTPUT_LONG)
#            define PTG_OUTPUT_LONG(file,param)   _PTGPrintLong(file,param)
#    endif
#    if !defined(PTG_OUTPUT_SHORT)
#            define PTG_OUTPUT_SHORT(file,param)  _PTGPrintInt(file,(int)param)
#    endif
#    if !defined(PTG_OUTPUT_CHAR)
#            define PTG_OUTPUT_CHAR(file,param)   _PTGPrintChar(file,param)
#    endif
#    if !defined(PTG_OUTPUT_FLOAT)
#            define PTG_OUTPUT_FLOAT(file,param)  _PTGPrintDouble(file,(double)param)
#    endif
#    if !defined(PTG_OUTPUT_DOUBLE)
#            define PTG_OUTPUT_DOUBLE(file,param) _PTGPrintDouble(file,param)
#    endif
#endif

/* Define PTGNode Type */
#if defined(__cplusplus)
struct _SPTG0;
typedef void (* _PTGProc)(struct _SPTG0 *);
#else
typedef void (* _PTGProc)();
#endif

typedef struct _SPTG0
{
  _PTGProc _print;
} * _PPTG0;
typedef _PPTG0 PTGNode;                      /* the only exported type */

/* predefined static PTGNULL-Node */

extern struct _SPTG0 _PTGNULL;
#define PTGNULL (& _PTGNULL)
#define PTGNull() (& _PTGNULL)

/* output functions */

#if PTG_OUTPUT_DEFAULT
extern PTGNode PTGOut ELI_ARG((PTGNode root));
extern PTGNode PTGOutFile ELI_ARG((char *filename, PTGNode root));
extern PTGNode PTGOutFPtr ELI_ARG((FILE *output, PTGNode root));
#endif
extern PTGNode PTGProcess ELI_ARG((PTG_OUTPUT_FILE file, PTGNode root));

/* Memory Management functions. */

extern void PTGFree ELI_ARG((void));

/* functions for making PTG nodes */

PTGNode PTGnode_definition ELI_ARG((char * p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5, PTGNode p6));
PTGNode PTGstatic_node_definition ELI_ARG((char * p1, PTGNode p2, PTGNode p3, PTGNode p4, PTGNode p5));
PTGNode PTGcomma_typename ELI_ARG((char * p1, int p2));
PTGNode PTGcomma_argument ELI_ARG((int p1));
PTGNode PTGsemicolon_typename ELI_ARG((char * p1, int p2));
PTGNode PTGassignment ELI_ARG((int p1));
PTGNode PTGvoid ELI_ARG((void));
PTGNode PTGnode_prototype ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGrefer_other_node ELI_ARG((char * p1, char * p2));
PTGNode PTGrefer_ptgnull ELI_ARG((char * p1));
PTGNode PTGHeaderFile ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGImplementationFile ELI_ARG((PTGNode p1));
PTGNode PTGstructure ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGstructure_component ELI_ARG((char * p1, int p2));
PTGNode PTGcheck_empty_argument ELI_ARG((int p1));
PTGNode PTGAndSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGcheck_empty_arguments ELI_ARG((PTGNode p1));
PTGNode PTGprint_function ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGprint_string ELI_ARG((char * p1));
PTGNode PTGprint_macro_call ELI_ARG((char * p1, int p2));
PTGNode PTGprint_invocation ELI_ARG((int p1));
PTGNode PTGprint_function_call ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGprint_function_argument ELI_ARG((int p1));
PTGNode PTGprint_opt_check ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGprint_optional_condition ELI_ARG((int p1));
PTGNode PTGfunction_call_prototype ELI_ARG((char * p1, PTGNode p2));
PTGNode PTGfunction_type ELI_ARG((char * p1));
PTGNode PTGnode_implementation ELI_ARG((char * p1, PTGNode p2, PTGNode p3, PTGNode p4));
PTGNode PTGnode_other ELI_ARG((char * p1, char * p2));
PTGNode PTGId ELI_ARG((int p1));
PTGNode PTGAsIs ELI_ARG((char * p1));
PTGNode PTGNumb ELI_ARG((int p1));
PTGNode PTGCString ELI_ARG((char * p1));
PTGNode PTGCChar ELI_ARG((int p1));
PTGNode PTGPString ELI_ARG((char * p1));
PTGNode PTGSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGCommaSeq ELI_ARG((PTGNode p1, PTGNode p2));
PTGNode PTGEol ELI_ARG((PTGNode p1));

/* prototypes for used function call insertions */

void condindent ELI_ARG((PTG_OUTPUT_FILE));
void condindenton ELI_ARG((PTG_OUTPUT_FILE));
void condindentoff ELI_ARG((PTG_OUTPUT_FILE));
void PtgOutId ELI_ARG((PTG_OUTPUT_FILE, int));
void CPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));
void CPtgOutchar ELI_ARG((PTG_OUTPUT_FILE, int));
void PPtgOutstr ELI_ARG((PTG_OUTPUT_FILE, char *));

#ifdef MONITOR
/* Monitoring support for structured values */
#define DAPTO_RESULTPTGNode(n) DAPTO_RESULT_PTR(n)
#define DAPTO_ARGPTGNode(n) DAPTO_ARG_PTR(n, PTGNode)
#endif

#endif

